@@ -65,7 +65,8 @@ qpwgraph_canvas::qpwgraph_canvas ( QWidget *parent )
 	: QGraphicsView(parent), m_state(DragNone), m_item(nullptr),
 		m_connect(nullptr), m_rubberband(nullptr),
 		m_zoom(1.0), m_zoomrange(false),
-		m_commands(nullptr), m_settings(nullptr), m_patchbay(nullptr),
+		m_commands(nullptr), m_settings(nullptr),
+		m_patchbay(nullptr), m_patchbay_edit(false),
 		m_selected_nodes(0), m_edit_item(nullptr),
 		m_editor(nullptr), m_edited(0)
 {
@@ -143,6 +144,98 @@ qpwgraph_patchbay *qpwgraph_canvas::patchbay (void) const
 }
 
 
+// Patchbay edit methods.
+void qpwgraph_canvas::setPatchbayEdit ( bool on )
+{
+	if (m_patchbay == nullptr)
+		return;
+
+	if ((!on && !m_patchbay_edit) ||
+		( on &&  m_patchbay_edit))
+		return;
+
+	foreach (QGraphicsItem *item, m_scene->items()) {
+		if (item->type() == qpwgraph_connect::Type) {
+			qpwgraph_connect *connect = static_cast<qpwgraph_connect *> (item);
+			if (connect)
+				connect->setDimmed(on && !m_patchbay->findConnect(connect));
+		}
+	}
+
+	m_patchbay_edit = on;
+}
+
+
+bool qpwgraph_canvas::isPatchbayEdit (void) const
+{
+	return (m_patchbay && m_patchbay_edit);
+}
+
+
+bool qpwgraph_canvas::canPatchbayPin (void) const
+{
+	if (m_patchbay == nullptr || !m_patchbay_edit)
+		return false;
+
+	foreach (QGraphicsItem *item, m_scene->selectedItems()) {
+		if (item->type() == qpwgraph_connect::Type) {
+			qpwgraph_connect *connect = static_cast<qpwgraph_connect *> (item);
+			if (connect && !m_patchbay->findConnect(connect))
+				return true;
+		}
+	}
+
+	return false;
+}
+
+
+bool qpwgraph_canvas::canPatchbayUnpin (void) const
+{
+	if (m_patchbay == nullptr || !m_patchbay_edit)
+		return false;
+
+	foreach (QGraphicsItem *item, m_scene->selectedItems()) {
+		if (item->type() == qpwgraph_connect::Type) {
+			qpwgraph_connect *connect = static_cast<qpwgraph_connect *> (item);
+			if (connect && m_patchbay->findConnect(connect))
+				return true;
+		}
+	}
+
+	return false;
+}
+
+
+void qpwgraph_canvas::patchbayPin (void)
+{
+	if (m_patchbay == nullptr || !m_patchbay_edit)
+		return;
+
+	foreach (QGraphicsItem *item, m_scene->selectedItems()) {
+		if (item->type() == qpwgraph_connect::Type) {
+			qpwgraph_connect *connect = static_cast<qpwgraph_connect *> (item);
+			if (connect && m_patchbay->connect(connect, true))
+				connect->setDimmed(false);
+		}
+	}
+}
+
+
+void qpwgraph_canvas::patchbayUnpin (void)
+{
+	if (m_patchbay == nullptr || !m_patchbay_edit)
+		return;
+
+	foreach (QGraphicsItem *item, m_scene->selectedItems()) {
+		if (item->type() == qpwgraph_connect::Type) {
+			qpwgraph_connect *connect = static_cast<qpwgraph_connect *> (item);
+			if (connect && m_patchbay->connect(connect, false))
+				connect->setDimmed(true);
+		}
+	}
+}
+
+
 // Canvas methods.
 void qpwgraph_canvas::addItem ( qpwgraph_item *item )
 {
@@ -165,6 +258,14 @@ void qpwgraph_canvas::addItem ( qpwgraph_item *item )
 		if (port)
 			restorePort(port);
 	}
+	else
+	if (item->type() == qpwgraph_connect::Type) {
+		qpwgraph_connect *connect = static_cast<qpwgraph_connect *> (item);
+		if (connect) {
+			connect->setDimmed(m_patchbay_edit &&
+				m_patchbay && !m_patchbay->findConnect(connect));
+		}
+	}
 }
 
 
@@ -1120,9 +1221,10 @@ bool qpwgraph_canvas::restoreNode ( qpwgraph_node *node )
 		= m_settings->value('/' + node_key).toPointF();
 	m_settings->endGroup();
 
-	if (!node_pos.isNull())
-		node->setPos(node_pos);
+	if (node_pos.isNull())
+		return false;
 
+	node->setPos(node_pos);
 	return true;
 }
 
