@@ -229,6 +229,17 @@ qpwgraph_form::qpwgraph_form (
 	// shortcuts firmly attached...
 	addAction(m_ui.viewMenubarAction);
 
+	// HACK: Make old Ins/Del standard shortcuts
+	// for connect/disconnect available again...
+	QList<QKeySequence> shortcuts;
+	shortcuts.append(m_ui.graphConnectAction->shortcut());
+	shortcuts.append(QKeySequence("Ins"));
+	m_ui.graphConnectAction->setShortcuts(shortcuts);
+	shortcuts.clear();
+	shortcuts.append(m_ui.graphDisconnectAction->shortcut());
+	shortcuts.append(QKeySequence("Del"));
+	m_ui.graphDisconnectAction->setShortcuts(shortcuts);
+
 	QObject::connect(m_ui.graphConnectAction,
 		SIGNAL(triggered(bool)),
 		m_ui.graphCanvas, SLOT(connectItems()));
@@ -260,6 +271,16 @@ qpwgraph_form::qpwgraph_form (
 		SIGNAL(toggled(bool)),
 		SLOT(patchbayExclusive(bool)));
 
+	QObject::connect(m_ui.patchbayEditAction,
+		SIGNAL(toggled(bool)),
+		SLOT(patchbayEdit(bool)));
+	QObject::connect(m_ui.patchbayPinAction,
+		SIGNAL(triggered(bool)),
+		SLOT(patchbayPin()));
+	QObject::connect(m_ui.patchbayUnpinAction,
+		SIGNAL(triggered(bool)),
+		SLOT(patchbayUnpin()));
+
 	QObject::connect(m_ui.patchbayScanAction,
 		SIGNAL(triggered(bool)),
 		SLOT(patchbayScan()));
@@ -603,6 +624,30 @@ void qpwgraph_form::patchbayExclusive ( bool on )
 }
 
 
+void qpwgraph_form::patchbayEdit ( bool on )
+{
+	m_ui.graphCanvas->setPatchbayEdit(on);
+
+	stabilize();
+}
+
+
+void qpwgraph_form::patchbayPin (void)
+{
+	m_ui.graphCanvas->patchbayPin();
+
+	stabilize();
+}
+
+
+void qpwgraph_form::patchbayUnpin (void)
+{
+	m_ui.graphCanvas->patchbayUnpin();
+
+	stabilize();
+}
+
+
 void qpwgraph_form::patchbayScan (void)
 {
 	qpwgraph_patchbay *patchbay = m_ui.graphCanvas->patchbay();
@@ -1011,6 +1056,9 @@ void qpwgraph_form::stabilize (void)
 	m_ui.patchbayScanAction->setEnabled(is_activated);
 	m_ui.patchbaySaveAction->setEnabled(is_dirty);
 
+	m_ui.patchbayPinAction->setEnabled(canvas->canPatchbayPin());
+	m_ui.patchbayUnpinAction->setEnabled(canvas->canPatchbayUnpin());
+
 	m_ui.graphConnectAction->setEnabled(canvas->canConnect());
 	m_ui.graphDisconnectAction->setEnabled(canvas->canDisconnect());
 
@@ -1208,6 +1256,11 @@ void qpwgraph_form::contextMenuEvent ( QContextMenuEvent *event )
 	stabilize();
 
 	QMenu menu(this);
+	if (m_ui.graphCanvas->isPatchbayEdit()) {
+		menu.addAction(m_ui.patchbayPinAction);
+		menu.addAction(m_ui.patchbayUnpinAction);
+		menu.addSeparator();
+	}
 	menu.addAction(m_ui.graphConnectAction);
 	menu.addAction(m_ui.graphDisconnectAction);
 	menu.addSeparator();
